
--------------------------------------------------------
--                  Initialize Addon                  --
--------------------------------------------------------

AddonName = "Broker_EpicCount"
EpicCount = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName)

--------------------------------------------------------
--                Initialize Variables                --
--------------------------------------------------------

--		 Options to set
-- Show each item quality, yes/no {poor, common, uncommon, rare, epic, legendary, heirloom}
-- Show 0's if item is empty for each quality

local options = {
	name = AddonName,
	handler = EpicCount,
	type = "group",
	args = {
		zeroHeader = {
			name = L["Show Zero"],
			type = "header",
			order = 10,
		},
		zeroDesc = {
			name = L["This determines whether EpicCount will show a zero if there are no items of a given quality.  Normally, it only shows values if there is an item equipped of a given quality."],
			type = "description",
			order = 11,
		},
		showZero = {
			name = L["Show Zero"],
			desc = L["This determines whether EpicCount will show a zero if there are no items of a given quality.  Normally, it only shows values if there is an item equipped of a given quality."],
			type = "toggle",
			get = "isZeroShown",
			set = "setZeroShown",
			order = 12,
		},
		qualHeader = {
			name = L["Qualities to Show"],
			type = "header",
			order = 20,
		},
		qualDesc = {
			name = L["Options for showing or excluding items of a certain quality"],
			type = "description",
			order = 21,
		},
		ShowPoor = {
			name = L["Show Poor"],
			desc = L["Whether to show poor (grey) items"],
			type = "toggle",
			get = "isPoorShown",
			set = "setPoorShown",
			order = 25,
		},
		ShowCommon = {
			name = L["Show Common"],
			desc = L["Whether to show common (white) items"],
			type = "toggle",
			get = "isCommonShown",
			set = "setCommonShown",
			order = 26,
		},
		ShowUncommon = {
			name = L["Show Uncommon"],
			desc = L["Whether to show uncommon (green) items"],
			type = "toggle",
			get = "isUncommonShown",
			set = "setUncommonShown",
			order = 27,
		},
		ShowRare = {
			name = L["Show Rare"],
			desc = L["Whether to show rare (blue) items"],
			type = "toggle",
			get = "isRareShown",
			set = "setRareShown",
			order = 28,
		},
		ShowEpic = {
			name = L["Show Epic"],
			desc = L["Whether to show epic (purple) items"],
			type = "toggle",
			get = "isEpicShown",
			set = "setEpicShown",
			order = 29,
		},
		ShowLegendary = {
			name = L["Show Legendary"],
			desc = L["Whether to show legendary (orange) items"],
			type = "toggle",
			get = "isLegendaryShown",
			set = "setLegendaryShown",
			order = 30,
		},
		ShowHeirloom = {
			name = L["Show Heirloom"],
			desc = L["Whether to show heirloom (brown) items"],
			type = "toggle",
			get = "isHeirloomShown",
			set = "setHeirloomShown",
			order = 31,
		},
		specialHeader = {
			name = L["Special Items"],
			type = "header",
			order = 40,
		},
		specialDesc = {
			name = L["Options for including/excluding certain items"],
			type = "description",
			order = 41,
		},
		showShirt = {
			name = L["Show Shirt"],
			desc = L["Whether to include shirt quality in totals"],
			type = "toggle",
			get = "isShirtShown",
			set = "setShirtShown",
			order = 45,
		},
		showTabard = {
			name = L["Show Tabard"],
			desc = L["Whether to include tabard quality in totals"],
			type = "toggle",
			get = "isTabardShown",
			set = "setTabardShown",
			order = 46,
		},
	},
}

local defaults = {
	char = {
		zeroShown = false,
		poorShown = false,
		commonShown = true,
		uncommonShown = true,
		rareShown = true,
		epicShown = true,
		legendaryShown = true,
		heirloomShown = true,
		shirtShown = false,
		tabardShown = false,
		itemQuals = " ",
	},
}

--------------------------------------------------------
--               Ace3 Default Functions               --
--------------------------------------------------------

function EpicCount:OnInitialize()	-- Code to run when addon is first loaded
	-- Set up saved variable databases for profile and character
	self.db = LibStub("AceDB-3.0"):New("EpicCountVars", defaults, true)
	
	-- Set up options frame
	LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName, options, {"epiccount", "ec"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName, AddonName)
	
	-- Set up Broker object
	self.ldbobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(AddonName, {
		type = "data source",
		-- Change startup text for more clarity - v2.3
		-- text = L["No Gear"],
		text = L["Not Loaded"],
		label = "EpicCount",
		icon = "Interface\\Icons\\INV_Enchant_VoidCrystal",
		OnClick = function(frame, btn)
			if btn == "LeftButton" then
				-- Show the character window
				ToggleCharacter("PaperDollFrame")
			elseif btn == "RightButton" then
				-- Go to interface options
				InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
			end
		end,
-- Will have more tooltip later
		OnTooltipShow = function(tooltip)
--			tooltip:AddLine("|cffffffffBroker_|r|cffa335eeEpic|r|cffffffffCount|r")
--			tooltip:AddLine(" ")
--			tooltip:AddLine("We'll get some more data here soon")
			self:EpicUpdate()
		end,
	})
end

function EpicCount:OnEnable()		-- Code to run when addon is enabled
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "EpicUpdate")
	self:EpicLoad()
	self:ScheduleTimer("EpicUpdate", 10)
end

function EpicCount:OnDisable()		-- Code to run when addon is disabled
end

--------------------------------------------------------
--                Load Data Function                  --
--------------------------------------------------------
function EpicCount:EpicLoad()
	if self.db.char.itemQuals ~= " " then
		self.ldbobj.text = self.db.char.itemQuals
	end
end

--------------------------------------------------------
--                Main Update Function                --
--------------------------------------------------------

function EpicCount:EpicUpdate()		-- Update the display
	-- Initialize variables
	local qualcount = {}	-- counts how many of each quality are equipped
	local text = " "		-- the text to display
	local slotID, blank		-- loop iteration variables (blank is dummy variable)
	local slashFlag = false	-- keep track of whether to display a slash or not
	local slots = {			-- slot list for loop
		"HeadSlot",
		"NeckSlot",
		"ShoulderSlot",
		"BackSlot",
		"ChestSlot",
		"ShirtSlot",
		"TabardSlot",
		"WristSlot",
		"HandsSlot", 
		"WaistSlot",
		"LegsSlot",
		"FeetSlot",
		"Finger0Slot",
		"Finger1Slot",
		"Trinket0Slot",
		"Trinket1Slot",
		"MainHandSlot",
		"SecondaryHandSlot",
-- Removed in 5.0.4
--		"RangedSlot",
	}
	
	-- Initialize qualcount array first
	for i = 0, 7 do
		qualcount[i] = 0
	end
	
	-- Iterate over character's gear
	for _,slotname in pairs(slots) do
		slotID, blank = GetInventorySlotInfo(slotname)
		local quality = GetInventoryItemQuality("player", slotID)
		-- if this piece exists, then add to array
		if quality then
			if slotname == "TabardSlot" and self.db.char.tabardShown then	-- only show tabard if they want
				qualcount[quality] = qualcount[quality]+1
			elseif slotname == "ShirtSlot" and self.db.char.shirtShown then	-- only show shirt if they want
				qualcount[quality] = qualcount[quality]+1
			elseif slotname ~= "TabardSlot" and slotname ~= "ShirtSlot" then	-- show everything else
				qualcount[quality] = qualcount[quality]+1
			end
		end
	end
	
	-- Create text display (this loop only shows values that exist)
	for i = 0, 7 do
		local r, g, b, hex = GetItemQualityColor(i)
		local itCounts = self.db.char.zeroShown or qualcount[i] ~= 0			-- decides based on options whether to include this value

		if i == 0 then									-- iterate over options and make sure we should show each quality
			itCounts = itCounts and self.db.char.poorShown
		elseif i == 1 then
			itCounts = itCounts and self.db.char.commonShown
		elseif i == 2 then
			itCounts = itCounts and self.db.char.uncommonShown
		elseif i == 3 then
			itCounts = itCounts and self.db.char.rareShown
		elseif i == 4 then
			itCounts = itCounts and self.db.char.epicShown
		elseif i == 5 then
			itCounts = itCounts and self.db.char.legendaryShown
		elseif i == 6 then
			itCounts = false 		-- these aren't currently in the game
		elseif i == 7 then
			itCounts = itCounts and self.db.char.heirloomShown
		end
		
		if slashFlag and itCounts then					-- there has been another value, and this one has a value as well...
			text = text .. "/"							-- (or zeros are shown) so display a slash separator first
		end
		
		if itCounts then								-- since there is a value (or zeros are shown)
			text = text .. "|c" .. hex .. qualcount[i] .. "|r"	-- display it (with color)
			slashFlag = true							-- and set a slash to be drawn before the next value
		end
	end
	
	-- Display values if they exist, or display default value
	if text == " " then
		self.ldbobj.text = " " .. L["No Gear"]
	else
		self.ldbobj.text = text
	end
	
	self.db.char.itemQuals = text						-- save the display for later
end

--------------------------------------------------------
--               OptionFrame Functions                --
--------------------------------------------------------
function EpicCount:isZeroShown(info)
	return self.db.char.zeroShown
end

function EpicCount:setZeroShown(info, value)
	self.db.char.zeroShown = value
	self:EpicUpdate()
end

function EpicCount:isPoorShown(info)
	return self.db.char.poorShown
end

function EpicCount:setPoorShown(info, value)
	self.db.char.poorShown = value
	self:EpicUpdate()
end

function EpicCount:isCommonShown(info)
	return self.db.char.commonShown
end

function EpicCount:setCommonShown(info, value)
	self.db.char.commonShown = value
	self:EpicUpdate()
end

function EpicCount:isUncommonShown(info)
	return self.db.char.uncommonShown
end

function EpicCount:setUncommonShown(info, value)
	self.db.char.uncommonShown = value
	self:EpicUpdate()
end

function EpicCount:isRareShown(info)
	return self.db.char.rareShown
end

function EpicCount:setRareShown(info, value)
	self.db.char.rareShown = value
	self:EpicUpdate()
end

function EpicCount:isEpicShown(info)
	return self.db.char.epicShown
end

function EpicCount:setEpicShown(info, value)
	self.db.char.epicShown = value
	self:EpicUpdate()
end

function EpicCount:isLegendaryShown(info)
	return self.db.char.legendaryShown
end

function EpicCount:setLegendaryShown(info, value)
	self.db.char.legendaryShown = value
	self:EpicUpdate()
end

function EpicCount:isHeirloomShown(info)
	return self.db.char.heirloomShown
end

function EpicCount:setHeirloomShown(info, value)
	self.db.char.heirloomShown = value
	self:EpicUpdate()
end

function EpicCount:isShirtShown(info)
	return self.db.char.shirtShown
end

function EpicCount:setShirtShown(info, value)
	self.db.char.shirtShown = value
	self:EpicUpdate()
end

function EpicCount:isTabardShown(info)
	return self.db.char.tabardShown
end

function EpicCount:setTabardShown(info, value)
	self.db.char.tabardShown = value
	self:EpicUpdate()
end
